@@ -104,8 +104,7 @@ def main():
     valid = arg.valid[0]
     model, eval_dataset = load_model(arg.model, model_dataset=valid, pretrained_path=arg.config)
     predict_parameter = load_predict_parameter(model, arg.enable_arg_panel)
-
-    if 'beamsearch' in predict_parameter and predict_parameter['beamsearch']:
+    if 'beamsearch' in predict_parameter and predict_parameter['beamsearch'] == 'True':
         eval_metrics = [EvalMetric(model.tokenizer) for _ in range(predict_parameter['beamsize'])]
     else:
         eval_metrics = [EvalMetric(model.tokenizer)]
@@ -148,7 +147,7 @@ def main():
     for eval_pos, eval_metric in enumerate(eval_metrics):
 
         argtype = "_dataset_" + valid.replace("/", "_").replace(".", "")
-        if 'beamsearch' in predict_parameter:
+        if 'beamsearch' in predict_parameter and predict_parameter['beamsearch'] == 'True':
             argtype = "_beam_" + str(eval_pos)
         outfile_name = arg.model + argtype
 
@@ -160,10 +159,15 @@ def main():
                 writer.writerow([i, p, "[SEP]".join([onet for onet in t if len(onet) > 0])])
         print("write result at:", outfile_name)
 
-        with open(outfile_name + "_score.csv", "w", encoding='utf8') as f:
-            for i in eval_metric.cal_score(arg.metric):
-                f.write("TASK: " + str(i[0]) + " , " + str(eval_pos) + '\n')
-                f.write(str(i[1]) + '\n')
+        print(eval_metric.cal_score(arg.metric))
+        with open(outfile_name + "_each_data_score.csv", "w", encoding='utf8') as edsf:
+            eds = csv.writer(edsf)
+            with open(outfile_name + "_score.csv", "w", encoding='utf8') as f:
+                for i in eval_metric.cal_score(arg.metric):
+                    f.write("TASK: " + str(i[0]) + " , " + str(eval_pos) + '\n')
+                    f.write(str(i[1]) + '\n')
+                    eds.writerows(i[2])
+
         print("write score at:", outfile_name)
 
         for i in eval_metric.cal_score(arg.metric):
